% -------------
% Get grid
% -------------
NX=32;
NZ=32;
gridfile='grid.h5';
namefile='start.h5';
amp=0.0001;
% -------------

y=h5read(gridfile,'/grids/y');
yf=0.5*(y(1:end-1)+y(2:end));
NY=length(yf);

UU=zeros([NX,NY,NZ]);
VV=zeros([NX,NY,NZ]);
WW=zeros([NX,NY,NZ]);
TTH1=zeros([NX,NY,NZ]);

% Baseflow
for yi=1:NY
    UU(1,yi,1)=1-yf(yi).^2;
    TTH1(1,yi,1)=yf(yi);
end

% Eigenmode
tmp=load('eig_u.dat');
ey=tmp(:,1);eu=tmp(:,2)+1j*tmp(:,3);
tmp=load('eig_v.dat');
ev=tmp(:,2)+1j*tmp(:,3);
tmp=load('eig_w.dat');
ew=tmp(:,2)+1j*tmp(:,3);

UU(2,:,1)=amp*interp1(ey,eu,yf,'spline');UU(NX,:,1)=conj(UU(2,:,1));
tmp=interp1(ey,ev,y,'spline');
VV(2,:,1)=amp*0.5*(tmp(1:end-1)+tmp(2:end));VV(NX,:,1)=conj(VV(2,:,1));
WW(2,:,1)=amp*interp1(ey,ew,yf,'spline');WW(NX,:,1)=conj(WW(2,:,1));

if exist(namefile,'file')
    eval(['!rm ' namefile])
end

h5create(namefile,'/Timestep/U'  ,[NX,NY,NZ]);
h5create(namefile,'/Timestep/V'  ,[NX,NY,NZ]);
h5create(namefile,'/Timestep/W'  ,[NX,NY,NZ]);
h5create(namefile,'/Timestep/TH1',[NX,NY,NZ]);

h5writeatt(namefile,'/','Resolution',int32([NX,NY,NZ]));
h5writeatt(namefile,'/Timestep','Time',0.)
% Take fft and write field
for yi=1:NY
    phys=ifftn(squeeze(UU(:,yi,:)))*NX*NZ;
    if max(max(max(abs(imag(phys)))))<1E-8;
        phys=reshape(real(phys),[NX,1,NZ]);
    end
    h5write(namefile,'/Timestep/U',phys,[1,yi,1],[NX,1,NZ]);
    phys=ifftn(squeeze(VV(:,yi,:)))*NX*NZ;
    if max(max(max(abs(imag(phys)))))<1E-8;
        phys=reshape(real(phys),[NX,1,NZ]);
    end
    h5write(namefile,'/Timestep/V',phys,[1,yi,1],[NX,1,NZ]);
    phys=ifftn(squeeze(WW(:,yi,:)))*NX*NZ;
    if max(max(max(abs(imag(phys)))))<1E-8;
        phys=reshape(real(phys),[NX,1,NZ]);
    end
    h5write(namefile,'/Timestep/W',phys,[1,yi,1],[NX,1,NZ]);
    phys=ifftn(squeeze(TTH1(:,yi,:)))*NX*NZ;
    if max(max(max(abs(imag(phys)))))<1E-8;
        phys=reshape(real(phys),[NX,1,NZ]);
    end
    h5write(namefile,'/Timestep/TH1',phys,[1,yi,1],[NX,1,NZ]);
end
